
#include <linux/types.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/init.h>
#include <linux/input.h>
#include <linux/nodemask.h>
#include <linux/platform_device.h>

#include <asm/hardware.h>
#include <asm/irq.h>
#include <asm/setup.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/irq.h>
#include <asm/mach/keypad.h>
#include <asm/arch/memory.h>
#include <asm/arch/gpio.h>
#include <asm/arch/clock.h>

#include "iomux.h"

#include <linux/mtd/mtd.h>
#include <linux/mtd/partitions.h>
#include <asm/mach/flash.h>
#include <asm/arch/clock.h>
/*#include "mxc_nd.h"*/
#include "crm_regs.h"

#define FLASH_NOR_BASE_ADDR	(0xa0000000)
#define FLASH_NOR_SIZE		(0x01000000) /* 16MB */


extern void mxc_map_io(void);
extern void mxc_init_irq(void);
extern void mxc_cpu_init(void) __init;
extern struct sys_timer mxc_timer;

extern void armadillo5x0_gpio_init(void) __init;
extern void mxc_cpu_common_init(void);


u32
get_machine_id(void)
{
	return 0;
}
EXPORT_SYMBOL(get_machine_id);

static struct resource armadillo5x0_smc911x_resources[] = {
	[0] = {
		.start	= 0xb2000000,
		.end	= 0xb2000000 + 0x02000000 - 1,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start	= 64,//INT_GPIO1,
		.end	= 64,//INT_GPIO1,
		.flags	= IORESOURCE_IRQ,
	},
};
struct smsc911x_platform_config {
	unsigned int irq_polarity;
	unsigned int irq_type;
};
static struct smsc911x_platform_config smsc911x_config = {
	.irq_polarity	= 0,
	.irq_type	= 1,
};
static struct platform_device armadillo5x0_smc911x_device = {
	.name		= "smsc911x",
	.id		= 0,
	.dev		= {
		.platform_data = &smsc911x_config,
	},
	.num_resources	= ARRAY_SIZE(armadillo5x0_smc911x_resources),  
	.resource	= armadillo5x0_smc911x_resources,
};

static void
armadillo5x0_eth_init(void)
{
	platform_device_register(&armadillo5x0_smc911x_device);
}

static struct mtd_partition mtd_armadillo5x0_nor_partitions[] = {
	{
	  .name		= "nor.bootloader",
	  .size		= 4 * 32 * 1024,
	  .offset	= 0x00000000,
	  .mask_flags	= MTD_WRITEABLE, /* force read-only */
	},
	{
	  .name		= "nor.kernel",
	  .size		= 16 * 128 * 1024,
	  .offset	= MTDPART_OFS_APPEND,
	  .mask_flags	= 0,
	},
	{
	  .name		= "nor.userland",
	  .size		= 110 * 128 * 1024,
	  .offset	= MTDPART_OFS_APPEND,
	  .mask_flags	= 0,
	},
	{
	  .name		= "nor.config",
	  .size		= 1 * 128 * 1024,
	  .offset	= MTDPART_OFS_APPEND,
	  .mask_flags	= 0,
	},
};

static struct flash_platform_data mtd_armadillo5x0_nor_data = {
	.map_name	= "cfi_probe",
	.width		= 2,
	.parts		= mtd_armadillo5x0_nor_partitions,
	.nr_parts	= ARRAY_SIZE(mtd_armadillo5x0_nor_partitions),
};

static struct resource mtd_armadillo5x0_nor_resource = {
	.start		= FLASH_NOR_BASE_ADDR,
	.end		= FLASH_NOR_BASE_ADDR + FLASH_NOR_SIZE - 1,
	.flags		= IORESOURCE_MEM,
};

static struct platform_device mtd_armadillo5x0_nor_device = {
	.name		= "armadillo5x0-nor",
	.id		= 0,
	.dev		= {
		.platform_data = &mtd_armadillo5x0_nor_data,
	},
	.num_resources	= 1,
	.resource	= &mtd_armadillo5x0_nor_resource,
};

static struct mtd_partition mtd_armadillo5x0_nand_partitions[] = {
	{
	 .name = "nand.ipl",
	 .offset = 0,
	 .size = 128 * 1024},
	{
	 .name = "nand.kernel",
	 .offset = MTDPART_OFS_APPEND,
	 .size = 4 * 1024 * 1024},
	{
	 .name = "nand.userland",
	 .offset = MTDPART_OFS_APPEND,
	 .size = 22 * 1024 * 1024},
	{
	 .name = "nand.free",
	 .offset = MTDPART_OFS_APPEND,
	 .size = MTDPART_SIZ_FULL},
};

static struct flash_platform_data mtd_armadillo5x0_nand_data = {
	.parts = mtd_armadillo5x0_nand_partitions,
	.nr_parts = ARRAY_SIZE(mtd_armadillo5x0_nand_partitions),
	.width = 1,
};

static struct platform_device mtd_armadillo5x0_nand_device = {
	.name = "mxc_nand_flash",
	.id = 0,
	.dev = {
		.platform_data = &mtd_armadillo5x0_nand_data,
	},
};

static void
armadillo5x0_mtd_init(void)
{
	/* NOR */
	(void)platform_device_register(&mtd_armadillo5x0_nor_device);

	/* NAND */
	if (__raw_readl(MXC_CCM_RCSR) & MXC_CCM_RCSR_NF16B) {
		mtd_armadillo5x0_nand_data.width = 2;
	}
	(void)platform_device_register(&mtd_armadillo5x0_nand_device);
}

static struct platform_device armadillo5x0_fb_device = {
	.name	= "mxc_sdc_fb",
	.id	= 0,
	.dev	= {
		.platform_data		= "TV-VGA",
		.coherent_dma_mask	= 0xFFFFFFFF,
	},
};

static void
armadillo5x0_fb_init(void)
{
	platform_device_register(&armadillo5x0_fb_device);
}

static struct platform_device armadillo5x0_led_device = {
	.name	= "armadillo5x0_led",
	.id	= 0,
};

static void
armadillo5x0_led_device_init(void)
{
	platform_device_register(&armadillo5x0_led_device);
}

static struct platform_device armadillo5x0_gpio_device = {
	.name = "armadillo5x0_gpio",
	.id = 0,
};

static void
armadillo5x0_gpio_device_init(void)
{
	platform_device_register(&armadillo5x0_gpio_device);
}

static struct platform_device armadillo5x0_tactsw_device = {
	.name = "armadillo5x0_tactsw",
	.id = 0,
};

static void
armadillo5x0_tactsw_device_init(void)
{
	platform_device_register(&armadillo5x0_tactsw_device);
}

extern int mxc_clocks_init(void);

static void __init
armadillo5x0_init(void)
{
	mxc_cpu_common_init();
	mxc_clocks_init();
	mxc_gpio_init();

	armadillo5x0_gpio_init();
	armadillo5x0_eth_init();
	armadillo5x0_mtd_init();
	armadillo5x0_fb_init();
	armadillo5x0_led_device_init();
	armadillo5x0_gpio_device_init();
	armadillo5x0_tactsw_device_init();
}

#if defined(CONFIG_MACH_ARMADILLO500)
MACHINE_START(ARMADILLO5X0, "Armadillo-500")
	/* Maintainer: Atmark Techno, Inc. */
	.phys_io	= AIPS1_BASE_ADDR,
	.io_pg_offst	= ((AIPS1_BASE_ADDR_VIRT) >> 18) & 0xfffc,
	.boot_params	= PHYS_OFFSET + 0x00000100,
	.map_io		= mxc_map_io,
	.init_irq	= mxc_init_irq,
	.timer		= &mxc_timer,
	.init_machine	= armadillo5x0_init,
MACHINE_END
#endif
